@@ -2,7 +2,7 @@ import abc
 
 # For 2 and 3 compatibility
 # https://stackoverflow.com/questions/35673474/
-ABC = abc.ABCMeta('ABC', (object,), {'__slots__': ()})
+ABC = abc.ABCMeta("ABC", (object,), {"__slots__": ()})
 
 
 class BaseOfficeFile(ABC):
